==> xdvik-22.74.3.tar.gz: xdvik-22.74.3/texk/xdvik/xdvi-search.el <==
;;; (X)Emacs frontend to forward search with xdvi(k).
;;; Requires xdvi(k) >= 22.38.
;;; See the section on FORWARD SEARCH in the xdvi man page for more
;;; information on forward and reverse search.
;;;
;;; This file is available from: http://xdvi.sourceforge.net/xdvi-search.el
;;;
;;; Preliminary test version, tested with Emacs 20.4 to 21.2 and Xemacs 21.1
;;;
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or (at your option) any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;;;
;;;
;;; Usage:
;;;
;;; - Add this file to some place where emacs can find it
;;;   (e.g. put it into a directory ~/emacs/, which you add to your
;;;   load path by putting the following line into your .emacs file:
;;;   
;;;      (add-to-list 'load-path (expand-file-name "~/emacs/"))
;;;   
;;;   Then, add the following line to your .emacs file:
;;;   
;;;      (require 'xdvi-search)
;;;   
;;;   After compiling your .tex file with source specials activated, you should
;;;   be able to use
;;;      M-x xdvi-jump-to-line
;;;   to make xdvi jump to the current location of the cursor.
;;;   
;;;   You could also bind this command to a key, e.g. by adding
;;;   a binding to tex-mode-hook:
;;;   
;;;      (add-hook 'tex-mode-hook (lambda ()
;;;                                 (local-set-key "\C-c\C-j" 'xdvi-jump-to-line)))
;;;
;;;   (without AucTeX; use LaTeX-mode-hook instead of tex-mode-hook with AucTeX.
;;;   Note that with AucTeX, C-c C-j is already bound to LaTeX-insert-item, so
;;;   you might want to use a different key combination instead.)
;;;
;;;
;;; Please send bug reports, improvements etc. to
;;; <stefanulrich@users.sourceforge.net>.
;;;


(defgroup xdvi-search nil
      "Support for inverse search with (La)TeX and DVI previewers."
      :group 'languages)

(defcustom explicit-shell-file-name nil
  "*If non-nil, file name to use for explicitly requested inferior shell."
  :type '(choice (const :tag "nil" nil) file)
  :group 'xdvi-search)


(defcustom xdvi-bin "xdvi"
  "*Name of the xdvi executable."
  :type '(choice (const "xdvi") file)
  :group 'xdvi-search)

(defcustom xdvi-logfile "~/.xdvi-log"
  "*Write xdvi output to this file, or throw away output if set to nil."
  :type '(choice (const :tag "nil" nil) file)
  :group 'xdvi-search)

(defun xdvi-jump-to-line ()
  "Call xdvi to perform a `forward search' for current file and line number.
Xdvi needs three pieces of information for this:

- the current line number
- the name of the current input file
- the name of the master .tex file.

The master .tex file (we follow AucTeX's terminology here) is the same
as `\jobname' and determines the name of the DVI file.  It is either
obtained by using AucTeX's variable `TeX-master-file' or, in case
AucTeX is available, by calling `xdvi-master-file-name', a function
which tries to mimick AucTeX's behaviour to obtain this filename.

For the current input file name, we need to rely on the DVI viewer to
make a smart matching of path suffixes with the file names that
actually occur in the specials, since specials inserted by TeX's
`-src' option might contain absolute or relative paths, depending on
whether TeX is called with a relative or absolute filename. (Similarly
for srcltx.sty, which can't expand paths.)
"
  (interactive)
  (save-excursion
    (save-restriction
      (widen)
      (beginning-of-line 1)
      (let* (;;; current line in file.
	     ;;; count-lines yields different results at beginning and in the
             ;;; middle of a line, so we normalize by going to BOL first and
             ;;; then adding 1
	     (curr-line (+ 1 (count-lines (point-min) (point))))
	     ;;; name of master .tex file, to be used as .dvi basename:
	     (master-file (expand-file-name (if (fboundp 'TeX-master-file)
					     (TeX-master-file t)
					   (xdvi-get-masterfile (xdvi-master-file-name)))))
	     ;;; .dvi file name:
	     (dvi-file (concat (file-name-sans-extension master-file) ".dvi"))
	     ;;; current source file name, expanded to full path:
	     (filename (expand-file-name (buffer-file-name))))
	(if (not (file-exists-p dvi-file))
	    (message "File %s does not exist; do you need to run latex first?" dvi-file)
	    (save-window-excursion
	      ;;; obtain the type of I/O-redirection needed for current shell.
	      (let* ((default-shell
		       (or (and (boundp 'explicit-shell-file-name) explicit-shell-file-name)
			   (getenv "ESHELL")
			   (getenv "SHELL")
			   "/bin/sh"))
		     (shell-redirection
		      (cond ((string-match "/bin/t?csh" default-shell)
			     (list ">&" ""))
			    ;; bash/ksh/sh
			    ((list ">" "2>&1")))))
	        ;;; The reason for using `shell' instead of
	        ;;; `start-process' or `call-process' is that I haven't
	        ;;; found a way of viewing the stderr/stdout output of the
	        ;;; xdvi child process without having either xdvi
	        ;;; freezing, Emacs freezing, or Emacs killing the xdvi
	        ;;; child process (which is forked when running with
	        ;;; -sourceposition).  Making xdvi disassociate from the
	        ;;; controling terminal is clearly not an option either ...
		(cond ((string-match "XEmacs" emacs-version)
		       (shell))
		      ((< emacs-major-version 21)
		       (shell))
		      ((shell "*xdvi-shell*")))
		(comint-send-input)
		(insert xdvi-bin
			" -sourceposition '" (int-to-string curr-line) " " filename "' "
			dvi-file
			" "
			(car shell-redirection)
			" "
		      ;;; we could probably do without the logfile, but it's an easy
		      ;;; way for toggling between /dev/null and logging.
			(if xdvi-logfile
			    (let ((xdvi-logfile-fullname (expand-file-name xdvi-logfile)))
			      ;;; to avoid problems with `noclobber'
			      (if (file-exists-p xdvi-logfile-fullname)
				  (delete-file xdvi-logfile-fullname))
			      xdvi-logfile-fullname)
			  "/dev/null")
			" "
			(car (cdr shell-redirection))
			" &")
		(if xdvi-logfile
		    (message "Writing xdvi output to \"%s\"." xdvi-logfile))
		(comint-send-input))))))))


(defun xdvi-get-masterfile (file)
  "Small helper function for AucTeX compatibility.
Converts the special value t that TeX-master might be set to
into a real file name."
  (if (eq file t)
      (buffer-file-name)
    file))


(defun xdvi-master-file-name ()
  "Emulate AucTeX's TeX-master-file function.
Partly copied from tex.el's TeX-master-file and TeX-add-local-master."
  (if (boundp 'TeX-master)
      TeX-master
    (let ((master-file (read-file-name "Master file (default this file): ")))
      (if (y-or-n-p "Save info as local variable? ")
	  (progn
	    (goto-char (point-max))
	    (if (re-search-backward "^\\([^\n]+\\)Local Variables:" nil t)
		(let* ((prefix (if (match-beginning 1)
				   (buffer-substring (match-beginning 1) (match-end 1))
				 ""))
		      (start (point)))
		  (re-search-forward (regexp-quote (concat prefix "End:")) nil t)
		  (if (re-search-backward (regexp-quote (concat prefix "TeX-master")) start t)
		      ;;; if TeX-master line exists already, replace it
		      (progn
			(beginning-of-line 1)
			(kill-line 1))
		    (beginning-of-line 1))
		  (insert prefix "TeX-master: " (prin1-to-string master-file) "\n"))
	      (insert "\n%%% Local Variables: "
;;; mode is of little use without AucTeX ...
;;;		      "\n%%% mode: " (substring (symbol-name major-mode) 0 -5)
		      "\n%%% TeX-master: " (prin1-to-string master-file)
		      "\n%%% End: \n"))
	    (save-buffer)
	    (message "(local variables written.)"))
	(message "(nothing written.)"))
      (set (make-local-variable 'TeX-master) master-file))))


(provide 'xdvi-search)

;;; page break to avoid "Local variables entry is missing the prefix" error for previous stuff


;;; xdvi-search.el ends here

==> xdvik-22.76.1.tar.gz: xdvik-22.76.1/texk/xdvik/xdvi-search.el <==
;;; (X)Emacs frontend to forward search with xdvi(k).
;;; Requires xdvi(k) >= 22.38.
;;; See the section on FORWARD SEARCH in the xdvi man page for more
;;; information on forward and reverse search.
;;;
;;; This file is available from: http://xdvi.sourceforge.net/xdvi-search.el
;;;
;;; Preliminary test version, tested with Emacs 20.4 to 21.2 and Xemacs 21.1
;;;
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or (at your option) any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;;;
;;;
;;; Usage:
;;;
;;; - Add this file to some place where emacs can find it
;;;   (e.g. put it into a directory ~/emacs/, which you add to your
;;;   load path by putting the following line into your .emacs file:
;;;   
;;;      (add-to-list 'load-path (expand-file-name "~/emacs/"))
;;;   
;;;   Then, add the following line to your .emacs file:
;;;   
;;;      (require 'xdvi-search)
;;;   
;;;   After compiling your .tex file with source specials activated, you should
;;;   be able to use
;;;      M-x xdvi-jump-to-line
;;;   to make xdvi jump to the current location of the cursor.
;;;   
;;;   You could also bind this command to a key, e.g. by adding
;;;   a binding to tex-mode-hook:
;;;   
;;;      (add-hook 'tex-mode-hook (lambda ()
;;;                                 (local-set-key "\C-c\C-j" 'xdvi-jump-to-line)))
;;;
;;;   (without AucTeX; use LaTeX-mode-hook instead of tex-mode-hook with AucTeX.
;;;   Note that with AucTeX, C-c C-j is already bound to LaTeX-insert-item, so
;;;   you might want to use a different key combination instead.)
;;;
;;;
;;; Please send bug reports, improvements etc. to
;;; <stefanulrich@users.sourceforge.net>.
;;;


(defgroup xdvi-search nil
      "Support for inverse search with (La)TeX and DVI previewers."
      :group 'languages)

(defcustom explicit-shell-file-name nil
  "*If non-nil, file name to use for explicitly requested inferior shell."
  :type '(choice (const :tag "nil" nil) file)
  :group 'xdvi-search)


(defcustom xdvi-bin "xdvi"
  "*Name of the xdvi executable."
  :type '(choice (const "xdvi") file)
  :group 'xdvi-search)

(defun xdvi-jump-to-line ()
  "Call xdvi to perform a `forward search' for current file and line number.
Xdvi needs three pieces of information for this:

- the current line number
- the name of the current input file
- the name of the master .tex file.

The master .tex file (we follow AucTeX's terminology here) is the same
as `\jobname' and determines the name of the DVI file.  It is either
obtained by using AucTeX's variable `TeX-master-file' or, in case
AucTeX is available, by calling `xdvi-master-file-name', a function
which tries to mimick AucTeX's behaviour to obtain this filename.

For the current input file name, we need to rely on the DVI viewer to
make a smart matching of path suffixes with the file names that
actually occur in the specials, since specials inserted by TeX's
`-src' option might contain absolute or relative paths, depending on
whether TeX is called with a relative or absolute filename. (Similarly
for srcltx.sty, which can't expand paths.)
"
  (interactive)
  (save-excursion
    (save-restriction
      (widen)
      (beginning-of-line 1)
      (let* (;;; Current line in file.
	     ;;; `count-lines' yields different results at beginning and in the
             ;;; middle of a line, so we normalize by going to BOL first and
             ;;; then adding 1
	     (curr-line (+ 1 (count-lines (point-min) (point))))
	     ;;; name of master .tex file, to be used as DVI basename:
	     (master-file (expand-file-name (if (fboundp 'TeX-master-file)
					     (TeX-master-file t)
					   (xdvi-get-masterfile (xdvi-master-file-name)))))
	     ;;; DVI file name:
	     (dvi-file (concat (file-name-sans-extension master-file) ".dvi"))
	     ;;; For the -sourceposition argument, use the current
	     ;;; source file name, expanded to full path. Note: using
	     ;;; the full path only works reliably with current
	     ;;; versions of xdvik (>= 22.74.2). For other versions,
	     ;;; use
	     ;;; (filename (file-name-nondirectory (buffer-file-name)))
	     ;;; instead, as in older versions of xdvi-search.el.
	     (filename (expand-file-name (buffer-file-name))))
	(if (not (file-exists-p dvi-file))
	    (message "File %s does not exist; maybe you need to run latex first?" dvi-file)
	    (save-window-excursion
	      (setq proc
		    (start-process "xdvi" "*xdvi output*"
				   xdvi-bin
				   "-nofork"
				   "-sourceposition"
				   (concat (int-to-string curr-line) " " filename)
				   dvi-file))
	      (set-process-sentinel proc 'xdvi-process-sentinel)))))))

(defun xdvi-process-sentinel (process signal)
  (let ((retmsg (substring signal 0 -1)))
    (if (string-match "abnormally" retmsg)
	(progn
	  (message "XDvi: %s" retmsg)
	  (pop-to-buffer (process-buffer process)))
      (message "XDvi: %s" retmsg))))

(defun xdvi-get-masterfile (file)
  "Small helper function for AucTeX compatibility.
Converts the special value t that TeX-master might be set to
into a real file name."
  (if (eq file t)
      (buffer-file-name)
    file))


(defun xdvi-master-file-name ()
  "Emulate AucTeX's TeX-master-file function.
Partly copied from tex.el's TeX-master-file and TeX-add-local-master."
  (if (boundp 'TeX-master)
      TeX-master
    (let ((master-file (read-file-name "Master file (default this file): ")))
      (if (y-or-n-p "Save info as local variable? ")
	  (progn
	    (goto-char (point-max))
	    (if (re-search-backward "^\\([^\n]+\\)Local Variables:" nil t)
		(let* ((prefix (if (match-beginning 1)
				   (buffer-substring (match-beginning 1) (match-end 1))
				 ""))
		      (start (point)))
		  (re-search-forward (regexp-quote (concat prefix "End:")) nil t)
		  (if (re-search-backward (regexp-quote (concat prefix "TeX-master")) start t)
		      ;;; if TeX-master line exists already, replace it
		      (progn
			(beginning-of-line 1)
			(kill-line 1))
		    (beginning-of-line 1))
		  (insert prefix "TeX-master: " (prin1-to-string master-file) "\n"))
	      (insert "\n%%% Local Variables: "
;;; mode is of little use without AucTeX ...
;;;		      "\n%%% mode: " (substring (symbol-name major-mode) 0 -5)
		      "\n%%% TeX-master: " (prin1-to-string master-file)
		      "\n%%% End: \n"))
	    (save-buffer)
	    (message "(local variables written.)"))
	(message "(nothing written.)"))
      (set (make-local-variable 'TeX-master) master-file))))


(provide 'xdvi-search)

;;; page break to avoid "Local variables entry is missing the prefix" error for previous stuff


;;; xdvi-search.el ends here
